<div id="topbar" style="display: none">
	<div class="title">
		<!-- Don't put a space between the img and a. That will break the hover layout. -->
		<img src="{INLINE_IMAGE:arrow-up.png}" width="11" height="10" alt=""><a href="javascript:void(Mizuho.smoothlyScrollToToc())">{TITLE}</a>
	</div>
	<a href="javascript:void(0)" id="current_section"></a>
</div>